"""
Generates a new template tag and writes it to stdout.

Usage:
maketemplatetag template_tag_name callableToExecute a=b c=d e=f

e.g.
`maketemplatetag get_relevant_foos getFoos for=user in=location`
will generate a template tag that can be used like:
{% get_relevant_foos for user in location %}

defining `getFoos` is up to you; after the code has been generated
you'll see what you need to do much more easily than I can explain
in documentation.
"""

def subclass_templatetag(tag_name, query, query_args=None):
    classname = ''.join(i.title() for i in tag_name.split('_'))
    prep_dict = "{" + ', '.join('"%s":"%s"' % tuple(arg.split('=')) for arg in query_args) + "}"
    query_args = [arg.split('=')[1] for arg in query_args]
    init_args = ', '.join(query_args)

    unfolded_dict = ', '.join('%s=%s' % (arg,arg) for arg in query_args)

    subclass = """
from django import template

from djangohelpers.templatetags import TemplateTagNode

class %(classname)s(TemplateTagNode):

    noun_for = %(prep_dict)s

    def __init__(self, varname, %(init_args)s):
        TemplateTagNode.__init__(self, varname, %(unfolded_dict)s)

    def execute_query(self, %(init_args)s):
        return %(query)s(%(unfolded_dict)s)

register = template.Library()
register.tag('%(tag_name)s', %(classname)s.process_tag)
"""
    return subclass % locals()

def main():
    import sys
    sys.argv = sys.argv[1:]
    try:
        print subclass_templatetag(sys.argv[0], sys.argv[1], sys.argv[2:])
    except:
        print __doc__

if __name__ == '__main__':
    main()



from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.models import Group
from django.http import HttpResponseForbidden
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.utils.http import urlquote


class HttpDeleteMiddleware(object):
    def process_request(self, request):
        if not request.GET.has_key('delete'):
            return None
        if request.method == "GET":
            return render_to_response('djangohelpers/confirm_delete.html')
        if request.method == "POST":
            request.method = "DELETE"

def path_matches(current_path, paths_to_match):
    for exempt_path in paths_to_match:
        try:
            if current_path.startswith(exempt_path):
                return True
        except TypeError: # it wasn't a string object .. must be a regex
            if exempt_path.match(current_path):
                return True

    return False

class AuthRequirementMiddleware(object):
    def process_request(self, request):
        if request.user.is_authenticated():
            return None

        path = urlquote(request.get_full_path())           

        if path_matches(path, getattr(settings, 'ANONYMOUS_PATHS', [])):
            return None

        return HttpResponseRedirect('%s?%s=%s' % (
                settings.LOGIN_URL,
                REDIRECT_FIELD_NAME,
                path))

class GroupRequirementMiddleware(object):
    def process_request(self, request):
        path = urlquote(request.get_full_path())

        permission_locks = getattr(settings, 'GROUP_REQUIREMENTS_PER_PATH', {})
        first_match = path_matches(path, permission_locks.keys())

        if not first_match:
            return None

        required_permission = permission_locks[first_match]

        required_permission = Group.objects.get(name=required_permission)

        if required_permission in request.user.groups.all():
            return None

        if request.user.is_anonymous():
            return HttpResponseRedirect('%s?%s=%s' % (
                    settings.LOGIN_URL,
                    REDIRECT_FIELD_NAME,
                    path))


        location = getattr(settings, 'GROUP_REQUIREMENTS_REDIRECT', None)
        if location:
            return HttpResponseRedirect(location)

        return HttpResponseForbidden("Insufficient priviledges")
        



from django.http import HttpResponseNotAllowed
try:
    from django.template.response import TemplateResponse
except ImportError:
    from django.shortcuts import render_to_response
    from django.template import RequestContext
    def TemplateResponse(request, template, context=None, mimetype=None, status=None, content_type=None, current_app=None):
        return render_to_response(template, context, context_instance=RequestContext(request), mimetype=content_type or mimetype)

class rendered_with(object):
    """ use like
    @rendered_with('foo.html')
    def my_view_handler(request, ...):
      ...
    """
    def __init__(self, template_name, mimetype=None):
        self.template_name = template_name
        self.mimetype = mimetype
        
    def __call__(self, func):
        def rendered_func(request, *args, **kwargs):
            response = func(request, *args, **kwargs)
            if not isinstance(response, dict):
                # let inner function implicitly opt out from rendered_with
                # for explicit control of rendering, or redirects, etc
                return response
            items = response
            items.setdefault('template_name',self.template_name)
            items.setdefault('controller_name','%s/%s'%(func.__module__,func.__name__) )
            return TemplateResponse(request, self.template_name, items, mimetype=self.mimetype)
        return rendered_func

class allow_http(object):
    """ use like
    @allow_http("GET", "POST")
    def my_view_handler(request, ...):
      ...
    """

    def __init__(self, *methods):
        self.methods = methods

    def __call__(self, func):
        def inner(request, *args, **kwargs):
            if request.method not in self.methods:
                return HttpResponseNotAllowed(self.methods)
            return func(request, *args, **kwargs)
        return inner

def register_admin(model, exclude=[], also=[]):
    from django.contrib import admin

    class Admin(admin.ModelAdmin):
        list_display = [f.name for f in model._meta.fields if f.name not in exclude] + also

    admin.site.register(model, Admin)
